/* ***** BEGIN LICENSE BLOCK *****
* Version: MPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Mozilla Public License Version
* 1.1 (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is by Greg Yardley
*
* The Initial Developer of the Original Code is
*   Greg Yardley
* Portions created by the Initial Developer are Copyright (C) 2007
* the Initial Developer. All Rights Reserved.
*
* Contributor(s):
*   Trevor Hobson
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the MPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the MPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */

var objGlobal = {
	nsIFilePicker: Components.interfaces.nsIFilePicker,
	objectionPrefs: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.objection."),
	objectStrings: '',
	optionflashAppDir: '',
	lsos: [],
	deletedLsos: [],
	count: 0,
	lsosTree: null,
	lastLsoSortColumn: '',  
	lastLsoSortAscending: false,

	sortOrderTree: {
		parent: ['parent', 'name', 'size'],
		name: ['name', 'parent', 'size'],
		size: ['size', 'parent', 'name']
	},

	lsosTreeView:
	{
		rowCount: 0,
		setTree: function(tree) {},
		getImageSrc: function(row, column) {},
		getProgressMode: function(row, column) {},
		getCellValue: function(row, column) {},
		getCellText: function(row, column) {
			var rv="";
			if (column.id=="parentCol") // Added .id TH
			{
				rv = objGlobal.lsos[row].parent;
			}
			else if (column.id=="nameCol") // Added .id TH
			{
				rv = objGlobal.lsos[row].name;
			}
			else if (column.id=="sizeCol")
			{
				rv = objGlobal.lsos[row].size + " bytes";
			}
			return rv;
		},
		isSeparator: function(index) {return false;},
		isSorted: function() {return false;},
		isContainer: function(index) {return false;},
		cycleHeader: function(aColId, aElt) {},
		getRowProperties: function(row, props) {},
		getColumnProperties: function(column, props) {},
		getCellProperties: function(row, column, props) {},
	},

	objection_init: function()
	{
		// Read string bundle set
		objGlobal.objectStrings = document.getElementById('lsoStrings');

		// Reference the tree
		objGlobal.lsosTree = document.getElementById('lsosTree');

		// Display error if selected directory does not exist
		var checkFlashDir = objGlobal.GetFlashDir();
		if (checkFlashDir == "")
		{
			document.getElementById("appdir").value = objGlobal.objectStrings.getString("dirNotFound");
			document.getElementById("objectionPreferences").showPane(document.getElementById("paneFolder"));
		}
		else
		{
			document.getElementById("appdir").value = checkFlashDir.path;
			objGlobal.loadLsos(checkFlashDir);
		}
	},

	changeAppDir: function()
	{
		var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(objGlobal.nsIFilePicker);
		fp.init(window, objGlobal.objectStrings.getString("selectDirTitle"), objGlobal.nsIFilePicker.modeGetFolder);
		var rv = fp.show();
		if (rv == objGlobal.nsIFilePicker.returnOK)
		{
			var file = fp.file;
			if (document.getElementById("appdir").value != file.path)
			{
				document.getElementById("appdir").value = file.path;
				objGlobal.objectionPrefs.setCharPref("flashAppDir", file.path);
				objGlobal.loadLsos(file);
			}
		}
	},

	searchAutomatic: function()
	{
		// Display error if selected directory does not exist
		var checkFlashDir = objGlobal.GetFlashDir(true);
		if (checkFlashDir == "")
		{
			document.getElementById("appdir").value = objGlobal.objectStrings.getString("dirNotFound");;
		}
		else
		{
			if (document.getElementById("appdir").value != checkFlashDir.path)
			{
				document.getElementById("appdir").value = checkFlashDir.path;
				objGlobal.lastLsoSortAscending = false; // reset sort order
				objGlobal.loadLsos(checkFlashDir);
			}
		}
	},

	Lso: function(number, file, name, size, path, parent, pathFlash)
	{
		this.number = number;
		this.file = file;
		this.name = name;
		this.size = size;
		this.path = path;
		this.pathShort = '...' + path.substr(pathFlash.length);
		this.parent = parent;
		return this;
	},

	loadLsos: function(flashDir)
	{
		objGlobal.count = 0;
		objGlobal.lsos = new Array();
		objGlobal.loadLsosSub(flashDir, flashDir.path);
		objGlobal.lsosTreeView.rowCount = objGlobal.lsos.length;
		objGlobal.lastLsoSortAscending = objTree.SortTreeArray(objGlobal.lsos, 'parent', objGlobal.lastLsoSortColumn, objGlobal.lastLsoSortAscending);
		objGlobal.lastLsoSortColumn = 'parent';
		objGlobal.lsosTree.view = objGlobal.lsosTreeView;
		objGlobal.lsosTree.treeBoxObject.invalidate();
		objGlobal.ClearLsoProperties();
		document.getElementById("removeAllLsos").disabled = objGlobal.lsos.length == 0;
	},

	loadLsosSub: function(currentDir, flashPath)
	{
		// Get directory entries, try prevents error if user does not have appropriate privilages to directory
		try
		{
			var entries = currentDir.directoryEntries;
		} catch(er)
		{
			return;
		}
		while(entries.hasMoreElements())
		{
			var entry = entries.getNext();
			entry.QueryInterface(Components.interfaces.nsIFile);
			if (entry.isFile())
			{
				var leaf  = entry.leafName;
				var dotIndex  = leaf.lastIndexOf('.');
				var extension = (dotIndex >= 0) ? leaf.substring(dotIndex+1) : "";
				if ((extension == "sol")||(extension == "fso"))
				{
					if (entry.path.search(/#SharedObjects/i) > -1)
					{
						var lsoSiteTry = objGlobal.lsoGetSite(entry.path);
						var lsoSite = (lsoSiteTry.length >=0) ? lsoSiteTry : entry.parent.leafName;
					}
					else
					{
						var lsoSite = entry.parent.leafName;
					}
					objGlobal.lsos[objGlobal.count] = new objGlobal.Lso(objGlobal.count++, entry, entry.leafName, entry.fileSize, entry.path, lsoSite, flashPath);
				}
			}
			else if (entry.isDirectory())
			{
				objGlobal.loadLsosSub(entry, flashPath);
			}
		}
	},

	lsoGetSite: function(filePath)
	{
		try
		{
			var filePathString = filePath.toString();
			var pathSplit = filePathString.match(/.(?=#SharedObjects)/i);
			if (pathSplit == '\\')
			{
				pathSplit = '\\\\';
			}
			var pathStartMatchString = '(?:.*#SharedObjects' + pathSplit + '[^' + pathSplit + ']*' + pathSplit + ')';
			var pathStartMatchRe = new RegExp(pathStartMatchString, 'i');
			var pathStart = filePathString.substr(0).match(pathStartMatchRe);
			var pathEndMatchString = '(?:.*#SharedObjects' + pathSplit + '[^' + pathSplit + ']*' + pathSplit + ')[^' + pathSplit + ']*';
			var pathEndMatchRe = new RegExp(pathEndMatchString, 'i');
			var pathEnd = filePathString.match(pathEndMatchRe);
			var lsoSiteTry = filePathString.substring(pathStart.toString().length, pathEnd.toString().length);
		} catch(er)
		{
			lsoSiteTry = '';
		}
		return lsoSiteTry;
	},

	LsoColumnSort: function(column)
	{
		objGlobal.lastLsoSortAscending = objTree.SortTree(objGlobal.lsosTree, objGlobal.lsosTreeView, objGlobal.lsos, column, objGlobal.lastLsoSortColumn, objGlobal.lastLsoSortAscending, true);
		objGlobal.lastLsoSortColumn = column;
	},

	LsoSelected: function()
	{
		var selections = objTree.GetTreeSelections(objGlobal.lsosTree);
		if (selections.length)
		{
			document.getElementById("removeLso").removeAttribute("disabled");
		}
		else
		{
		return true;
		}
		var indexvar = selections[0];
		if (indexvar >= objGlobal.lsos.length)
		{
			// Something got out of synch.  See bug 119812 for details (Included from original code, bug has been fixed but keep to be safe - TH)
			dump("Out of synch;see Mozilla bug 119812 to help.");
			return;
		}
		if (selections.length > 1)
		{
			objGlobal.ClearLsoProperties();
		}
		else
		{
			objGlobal.SetLsoProperties(objGlobal.lsos[indexvar].name, objGlobal.lsos[indexvar].parent, objGlobal.lsos[indexvar].size + " bytes", objGlobal.lsos[indexvar].pathShort, objGlobal.lsos[indexvar].file);
		}
		return true;
	},

	HandleLsoKeyPress: function(e)
	{
		if (e.keyCode == 46)
		{
			objGlobal.DeleteLso();
		}
	},

	DeleteLso: function()
	{
		if (confirm(objGlobal.objectStrings.getString("confirmLsoRemove")))
		{
			var toDelete = objTree.GetTreeSelections(objGlobal.lsosTree);
			for (var x=0;x<toDelete.length;x++)
			{
				objGlobal.lsos[toDelete[x]].file.remove(false);
			}
			objTree.DeleteSelectedItemFromTree(objGlobal.lsosTree, objGlobal.lsosTreeView, objGlobal.lsos, objGlobal.deletedLsos, "removeLso", "removeAllLsos");
			if (!objGlobal.lsos.length)
			{
				objGlobal.ClearLsoProperties();
			}
		}
	},

	DeleteAllLsos: function()
	{
		if (confirm(objGlobal.objectStrings.getString("confirmLsoRemoveAll")))
		{
			for (var x=0;x<objGlobal.lsos.length;x++)
			{
				objGlobal.lsos[x].file.remove(false);
			}
			objTree.DeleteAllFromTree(objGlobal.lsosTree, objGlobal.lsosTreeView, objGlobal.lsos, objGlobal.deletedLsos, "removeLso", "removeAllLsos");
			objGlobal.ClearLsoProperties();
		}
	},

	ClearLsoProperties: function()
	{
		objGlobal.SetLsoProperties("", "", "", "", "");
	},

	SetLsoProperties: function(lso_name, lso_parent, lso_size, lso_path, lso_file)
	{
// Am just going to leave this commented out for the moment, people may want it back
//		document.getElementById("lso_name").value = lso_name;
//		document.getElementById("lso_parent").value = lso_parent;
//		document.getElementById("lso_size").value = lso_size;
		document.getElementById("lso_path").value = lso_path;
		objLSO.init(lso_file);

	},

	// automatic detection for Flash Apps directory
	GetFlashDir: function(ForceSearch)
	{
		var objectionPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.objection.");
		var LocalFile = new Components.Constructor("@mozilla.org/file/local;1", "nsILocalFile");
		var flashDir = new LocalFile();
		var testDir = new LocalFile();
		var varDir = new LocalFile();
		if (((!objectionPrefs.prefHasUserValue("flashAppDir"))||(typeof(objectionPrefs.getCharPref("flashAppDir")) == 'undefined'))||ForceSearch)
		{
			if (window.navigator.platform == "Win32")
			{
				var winDir = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("AppData", Components.interfaces.nsILocalFile);
				flashDir.initWithFile(winDir);
				testDir.initWithFile(flashDir);
				testDir.appendRelativePath('Macromedia');
				if ((testDir.exists()) && (testDir.isDirectory()))
				{
					flashDir.initWithFile(testDir);
					testDir.appendRelativePath('Flash Player');
					if ((testDir.exists()) && (testDir.isDirectory()))
					{
						flashDir.initWithFile(testDir);
					}
					varDir.initWithFile(flashDir);
					objectionPrefs.setCharPref("flashAppDir", varDir.path);

				}
				else
				{
					varDir = "";
					objectionPrefs.setCharPref("flashAppDir", "");
				}
			}
			else if (window.navigator.platform == "MacPPC")
			{
				var macDir = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("UsrPrfs", Components.interfaces.nsILocalFile);
				flashDir.initWithFile(macDir);
				testDir.initWithFile(flashDir);
				testDir.appendRelativePath('Macromedia');
				if ((testDir.exists()) && (testDir.isDirectory()))
				{
					flashDir.initWithFile(testDir);
					testDir.appendRelativePath('Flash Player');
					if ((testDir.exists()) && (testDir.isDirectory()))
					{
						flashDir.initWithFile(testDir);
					}
					varDir.initWithFile(flashDir);
					objectionPrefs.setCharPref("flashAppDir", varDir.path);

				}
				else
				{
					varDir = "";
					objectionPrefs.setCharPref("flashAppDir", "");
				}
			}
			else
			{
				var homeDir = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("Home", Components.interfaces.nsILocalFile);
				flashDir.initWithFile(homeDir);
				testDir.initWithFile(flashDir);
				testDir.appendRelativePath('.macromedia');
				if ((testDir.exists()) && (testDir.isDirectory()))
				{
					flashDir.initWithFile(testDir);
					testDir.appendRelativePath('Macromedia');
					if ((testDir.exists()) && (testDir.isDirectory()))
					{
						flashDir.initWithFile(testDir);
					}
					testDir.initWithFile(flashDir);
					testDir.appendRelativePath('Flash_Player');
					if ((testDir.exists()) && (testDir.isDirectory()))
					{
						flashDir.initWithFile(testDir);
					}
					else
					{
						testDir.initWithFile(flashDir);
						testDir.appendRelativePath('Flash Player');
						if ((testDir.exists()) && (testDir.isDirectory()))
						{
							flashDir.initWithFile(testDir);
						}
						else
						{
							testDir.initWithFile(flashDir);
							testDir.appendRelativePath('Flash \Player');
							if ((testDir.exists()) && (testDir.isDirectory()))
							{
								flashDir.initWithFile(testDir);
							}
						}
					}
					varDir.initWithFile(flashDir);
					objectionPrefs.setCharPref("flashAppDir", varDir.path);

				}
				else
				{
					varDir = "";
					objectionPrefs.setCharPref("flashAppDir", "");
				}
			}
		}
		else
		{
			if (objectionPrefs.getCharPref("flashAppDir") != "")
			{
				varDir.initWithPath(objectionPrefs.getCharPref("flashAppDir"));
			}
			else varDir = "";
		}
		return varDir;
	},
};
