// View LSO data
// Ported from http://osflash.org/s2x (Public Domain, No Licencing issues.)
// by Trevor Hobson
var objLSO = {
	NUMBER: '\x00',
	BOOLEAN: '\x01',
	STRING: '\x02',
	OBJOBJ: '\x03',
	NULL: '\x05',
	UNDEF: '\x06',
	OBJARR: '\x08',
	//RAWARR: '\x0A',
	OBJDATE: '\x0B',
	OBJM: '\x0D',
	OBJXML: '\x0F',
	OBJCC: '\x10',
	contentArray: [],
	count: 0,
	contentTree: null,

	contentTreeView:
	{
		rowCount: 0,
		setTree: function(treeBox) {this.treeBox = treeBox;},
		getImageSrc: function(row, column) {},
		getProgressMode: function(row, column) {},
		getCellValue: function(row, column) {},
		getCellText: function(row, column) {
			var rv="";
			if (column.id=="nameCol")
			{
				rv = objLSO.contentArray[row].name;
			}
			else if (column.id=="typeCol")
			{
				rv = objLSO.contentArray[row].type;
			}
			else if (column.id=="valueCol")
			{
				rv = objLSO.contentArray[row].value;
			}
			return rv;
		},
		getParentIndex: function(row) {
			if (objLSO.contentArray[row].level > 0)
			{
				for (var t=row-1; t>=0; t--)
				{
					if (objLSO.contentArray[t].level < objLSO.contentArray[row].level)
					{
						return t;
					}
				}
			}
			else
			{
				return -1;
			}
		},
		getLevel: function(row) {
			return objLSO.contentArray[row].level;
		},
		isSeparator: function(index) {return false;},
		isSorted: function() {return false;},
		isContainer: function(row) {return false;}, // Say false because we don't want the twistie
		isContainerOpen: function(row) {return true;},
		isContainerEmpty: function(row) {return false;},
		isEditable: function(row, column) {return false;},
		hasNextSibling: function(row, after) {
			var thisLevel = objLSO.contentArray[row].level;
			for (var t = row+1; t < objLSO.contentArray.length; t++)
			{
				var nextLevel = objLSO.contentArray[t].level;
				if (nextLevel == thisLevel)
				{
					return true;
				}
				else if (nextLevel < thisLevel)
				{
					return false;
				}
			}
		},
		toggleOpenState: function(row) {},
		cycleHeader: function(aColId, aElt) {},
		selectionChanged: function() {},
		cycleCell: function(row, column) {},
		performAction: function(action) {},
		performActionOnCell: function(action, row, column) {},
		getRowProperties: function(row, props) {},
		getColumnProperties: function(column, props) {},
		getCellProperties: function(row, column, props) {},
	},

	init: function(lsoFile)
	{
		// Make sure a file was passed
		try
		{
			if (!lsoFile.isFile())
			{
				return;
			}
		} catch(er)
		{
			return;
		}
		// Reference the tree
		objLSO.contentTree = document.getElementById('contentTree');
		// Open the LSO and read it
		objLSO.count = 0;
		objLSO.contentArray = new Array();
//		try
//		{
			objLSO.readLso(lsoFile);
//		} catch(er) {}
		objLSO.contentTreeView.rowCount = objLSO.contentArray.length;
		objLSO.contentTree.view = objLSO.contentTreeView;
		objLSO.contentTree.treeBoxObject.invalidate();
	},

	readLso: function(lsoFile)
	{
		var istream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
		istream.init(lsoFile, -1, -1, false);

		var bstream = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(Components.interfaces.nsIBinaryInputStream);
		bstream.setInputStream(istream);

		var nLenFile = bstream.available();
		var sHeader = bstream.readBytes(2);
		var nLenData = bstream.read32(4); // (Unsigned) Long
		if (nLenFile != nLenData + 6)
		{
//			alert('Warning: Data Length Mismatch.'); // Just ignore in production, for testing only
			objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'error', 'Data Length Mismatch', 'error', 0, false);
			return;
		}
		var sFileType = bstream.readBytes(4); // 'TCSO'
		var sth1 = bstream.readBytes(6);
		var nLenSoln = bstream.read16(2); // (Unsigned) Short
		var solname = bstream.readBytes(nLenSoln);
		var sth2 = bstream.readBytes(4);
		objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'name', solname, '', treeLevel, false);
		var treeLevel = 0;
//		try
//		{
			while (bstream.available() > 0)
			{
				var nLenVname = bstream.read16(2); // (Unsigned) Short
				var sVname = bstream.readBytes(nLenVname);
				objLSO.readStream(bstream, sVname, treeLevel);
				xEnd = bstream.readBytes(1); // '\x00'
			}
//		} catch(er)
//		{
//			objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'error', sVname, 'error', treeLevel, false);
//			bstream.close();
//		}
		bstream.close();
	},

	readStream: function(bstream, sVname, treeLevel)
	{
		var xType = bstream.readBytes(1);
		if (xType == objLSO.NUMBER)
		{
			objLSO.num2x(bstream, sVname, treeLevel);
		}
		else if (xType == objLSO.BOOLEAN)
		{
			objLSO.bol2x(bstream, sVname, treeLevel);
		}
		else if (xType == objLSO.STRING)
		{
			objLSO.str2x(bstream, sVname, treeLevel);
		}
		else if (xType == objLSO.OBJOBJ)
		{
			objLSO.obj2x(bstream, sVname, treeLevel);
		}
		else if (xType == objLSO.OBJARR)
		{
			objLSO.arr2x(bstream, sVname, treeLevel);
		}
		else if (xType == objLSO.OBJDATE)
		{
			objLSO.dat2x(bstream, sVname, treeLevel);
		}
		else if (xType == objLSO.OBJXML)
		{
			objLSO.xml2x(bstream, sVname, treeLevel);
		}
		else if (xType == objLSO.OBJCC)
		{
			objLSO.occ2x(bstream, sVname, treeLevel);
		}
		else if (xType == objLSO.OBJM)
		{
			objLSO.ojm2x(bstream, sVname, treeLevel);
		}
		else if (xType == objLSO.NULL)
		{
			objLSO.nul2x(bstream, sVname, treeLevel);
		}
		else if (xType == objLSO.UNDEF)
		{
			objLSO.und2x(bstream, sVname, treeLevel);
		}
		else
		{
			var xTypeHex = xType.charCodeAt(0).toString(16)
			objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'error', sVname, xTypeHex, treeLevel, false);
		}
	},

	contentFrame: function(number, type, name, value, level, container)
	{
		this.number = number;
		this.type = type;
		this.name = name;
		this.value = value;
		this.level = level;
		this.container = container;
		return this;
	},

	num2x: function(bstream, sVname, treeLevel)
	{
		try
		{
			var s = bstream.readDouble(8); // Double
			var sValue = String(s);
		} catch(er)
		{
			var sValue = 'error';
		}
		objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'number', sVname, sValue, treeLevel, false);
	},

	bol2x: function(bstream, sVname, treeLevel)
	{
		var b = bstream.readBytes(1)
		if (b == '\x00')
		{
			var sValue = 'false';
		}
		else if (b == '\x01')
		{
			var sValue = 'true';
		}
		else
		{
			var sValue = 'err';
		}
		objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'boolean', sVname, sValue, treeLevel, false);
	},

	str2x: function(bstream, sVname, treeLevel)
	{
		var nLenStr = bstream.read16(2); // (Unsigned) Short
		var sValue = bstream.readBytes(nLenStr);
		objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'string', sVname, sValue, treeLevel, false);
	},

	obj2x: function(bstream, sVname, treeLevel)
	{
		objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'object', sVname, '', treeLevel, true);
		var nLenVname = bstream.read16(2); // Unsigned Short
		while (nLenVname != 0)
		{
			var sVname = bstream.readBytes(nLenVname);
			objLSO.readStream(bstream, sVname, treeLevel + 1);
			var nLenVname = bstream.read16(2); // (Unsigned) Short
		}
		xEndObj = bstream.readBytes(1); // '\x09'
	},

	arr2x: function(bstream, sVname, treeLevel)
	{
		var nArrLength = bstream.read32(4); // (Unsigned) Long
		objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'array', sVname, '', treeLevel, true);
		var nLenVname = bstream.read16(2); // (Unsigned) Short
		while (nLenVname != 0)
		{
			var sVname = bstream.readBytes(nLenVname);
			objLSO.readStream(bstream, sVname, treeLevel + 1);
			var nLenVname = bstream.read16(2); // (Unsigned) Short
		}
		xEndObj = bstream.readBytes(1); // '\x09'
	},

	dat2x: function(bstream, sVname, treeLevel)
	{
		var nMsec = bstream.readDouble(8); // Double
		var nMinOffset = bstream.read16(2); // (Signed) Short - Does not read negative as short so need to convert it
		if (nMinOffset > 0x7fff)
		{
			nMinOffset = ~(0x10000 - nMinOffset - 1);
		}
		var nOffset = nMinOffset/60;
		var sOffsetSign = (nOffset <= 0) ? '+' : ''; // Create a + for positive offset
		var sOffset = sOffsetSign + String(-nOffset);
		var nDate = new Date(Date.UTC(1969,12,1) + nMsec); // Create a date starting at Dec 1 1969
		var sDate = nDate.getUTCFullYear() + '-' + (nDate.getUTCMonth()+1) + '-' + nDate.getUTCDate() + ' ' + nDate.getUTCHours() + ':' + nDate.getUTCMinutes() + ':' + nDate.getUTCSeconds() + '.' + nDate.getUTCMilliseconds() + ' UTC ' + sOffset;
		objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'date', sVname, sDate, treeLevel, false);
	},

	xml2x: function(bstream, sVname, treeLevel)
	{
		var nLenCData = bstream.read32(4); // (Unsigned) Long
		var sCData = bstream.readBytes(nLenCData);
		objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'xml', sVname, sCData, treeLevel, false);
	},

	occ2x: function(bstream, sVname, treeLevel)
	{
		var nLenCname = bstream.read16(2); // (Unsigned) Short
		var sCname = bstream.readBytes(nLenCname);
		objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'c_object', sVname, sCname, treeLevel, true);
		var nLenVname = bstream.read16(2); // (Unsigned) Short
		while (nLenVname != 0)
		{
			var sVname = bstream.readBytes(nLenVname);
			objLSO.readStream(bstream, sVname, treeLevel + 1);
			var nLenVname = bstream.read16(2); // (Unsigned) Short
		}
		xEndObj = bstream.readBytes(1); // '\x09'
	},

	ojm2x: function(bstream, sVname, treeLevel)
	{
		objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'm_object', sVname, '', treeLevel, false);
	},

	nul2x: function(bstream, sVname, treeLevel)
	{
		objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'null', sVname, '', treeLevel, false);
	},

	und2x: function(bstream, sVname, treeLevel)
	{
		objLSO.contentArray[objLSO.count] = new objLSO.contentFrame(objLSO.count++, 'undefined', sVname, '', treeLevel, false);
	},
};
