/* ***** BEGIN LICENSE BLOCK *****
* Version: MPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Mozilla Public License Version
* 1.1 (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is Mozilla Communicator client code, released
* March 31, 1998.
*
* The Initial Developer of the Original Code is
* Netscape Communications Corporation.
* Portions created by the Initial Developer are Copyright (C) 1998
* the Initial Developer. All Rights Reserved.
*
* Contributor(s):
*   Ben "Count XULula" Goodger
*   Brian Ryner <bryner@brianryner.com>
*   Trevor Hobson
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the MPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the MPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */

// Code from firefox 1.5 passwordManager.js - Trevor Hobson

var objTree = {

	DeleteAllFromTree: function(tree, view, table, deletedTable, removeButton, removeAllButton)
	{
		// remove all items from table and place in deleted table
		for (var i=0; i<table.length; i++)
		{
			deletedTable[deletedTable.length] = table[i];
		}
		table.length = 0;

		// clear out selections
		view.selection.select(-1); 

		// update the tree view and notify the tree
		view.rowCount = 0;

		var box = tree.treeBoxObject;
		box.rowCountChanged(0, -deletedTable.length);
		box.invalidate();

		// disable buttons
		document.getElementById(removeButton).setAttribute("disabled", "true")
		document.getElementById(removeAllButton).setAttribute("disabled","true");
	},

	DeleteSelectedItemFromTree: function(tree, view, table, deletedTable, removeButton, removeAllButton)
	{
		var box = tree.treeBoxObject;

		// Remove selected items from list (by setting them to null) and place in
		// deleted list.  At the same time, notify the tree of the row count changes.
		var selection = box.view.selection;
		var oldSelectStart = table.length;
		box.beginUpdateBatch();

		var selCount = selection.getRangeCount();
		var min = new Object();
		var max = new Object();

		for (var s = 0; s < selCount; ++s)
		{
			selection.getRangeAt(s, min, max);
			var minVal = min.value;
			var maxVal = max.value;

			oldSelectStart = minVal < oldSelectStart ? minVal : oldSelectStart;

			var rowCount = maxVal - minVal + 1;
			view.rowCount -= rowCount;
			box.rowCountChanged(minVal, -rowCount);

			for (var i = minVal; i <= maxVal; ++i)
			{
				deletedTable[deletedTable.length] = table[i];
				table[i] = null;
			}
		}

		// collapse list by removing all the null entries
		for (var j = 0; j < table.length; ++j)
		{
			if (!table[j])
			{
				var k = j;
				while (k < table.length && !table[k])
				k++;
				table.splice(j, k-j);
			}
		}

		box.endUpdateBatch();

		// update selection and/or buttons
		var removeBtn = document.getElementById(removeButton);
		var removeAllBtn = document.getElementById(removeAllButton);

		if (table.length)
		{
			removeBtn.removeAttribute("disabled");
			removeAllBtn.removeAttribute("disabled");

			selection.select(oldSelectStart < table.length ? oldSelectStart : table.length - 1);
		}
		else
		{
			removeBtn.setAttribute("disabled", "true");
			removeAllBtn.setAttribute("disabled", "true");
		}
	},

	GetTreeSelections: function(tree)
	{
		var selections = [];
		var select = tree.view.selection;
		if (select)
		{
			var count = select.getRangeCount();
			var min = new Object();
			var max = new Object();
			for (var i=0; i<count; i++)
			{
				select.getRangeAt(i, min, max);
				for (var k=min.value; k<=max.value; k++)
				{
					if (k != -1)
					{
						selections[selections.length] = k;
					}
				}
			}
		}
		return selections;
	},

	// Split original into two functions (SortTree and SortTreeArray) so can sort the array before applying to tree - Trevor Hobson
	SortTree: function(tree, view, table, column, lastSortColumn, lastSortAscending, updateSelection)
	{
		// remember which item was selected so we can restore it after the sort
		var selections = objTree.GetTreeSelections(tree);
		var selectedNumber = selections.length ? table[selections[0]].number : -1;

		// sort the table (array)
		var ascending = objTree.SortTreeArray(table, column, lastSortColumn, lastSortAscending);

		// restore the selection
		var selectedRow = -1;
		if (selectedNumber>=0 && updateSelection)
		{
			for (var s=0; s<table.length; s++)
			{
				if (table[s].number == selectedNumber)
				{
					// update selection
					// note: we need to deselect before reselecting in order to trigger ...Selected()
					tree.view.selection.select(-1);
					tree.view.selection.select(s);
					selectedRow = s;
					break;
				}
			}
		}

		// display the results
		tree.treeBoxObject.invalidate();
		if (selectedRow >= 0)
		{
			tree.treeBoxObject.ensureRowIsVisible(selectedRow)
		}

		return ascending;
	},

	SortTreeArray: function(table, column, lastSortColumn, lastSortAscending)
	{
		// determine if sort is to be ascending or descending
		var ascending = (column == lastSortColumn) ? !lastSortAscending : true;

		// determine sub sort
		var sortOrder = objGlobal.sortOrderTree[column];

		// do the sort
		var compareFunc;
		if (ascending)
		{
			compareFunc = function compare(first, second)
			{
				return objTree.CompareArray(first, second, sortOrder);
			}
		}
		else
		{
			compareFunc = function compare(first, second)
			{
				return objTree.CompareArray(second, first, sortOrder);
			}
		}

		table.sort(compareFunc);
		return ascending;
	},

	// Changed from original CompareLowerCase, allows multiple column sort - Trevor Hobson
	CompareArray: function(first, second, sortOrder)
	{
		if (sortOrder.length == 0)
		{
			return 0;
		}

		var sortOrderSub = sortOrder.slice(1);
		var column = sortOrder[0];

		if (isNaN(first[column]) || isNaN(second[column]))
		{
			var x = first[column].toLowerCase();
			var y = second[column].toLowerCase();
		}
		else
		{
			var x = first[column];
			var y = second[column];
		}

		return ((x < y) ? -1 : ((x > y) ? 1 : objTree.CompareArray(first, second, sortOrderSub)));
	},
};